const BaseReporter = Mocha.reporters.Base
const EventConsts = Mocha.Runner.constants

const WebSkipedErrorCode = 9005
const AppSkipedErrorCode = 9002

class QAReporter extends BaseReporter {
  constructor(runner, options) {
    super(runner, options)

    const dataAnalyzer = options.reporterOptions.dataAnalyzer

    // 测试用例开始执行
    runner.on(EventConsts.EVENT_TEST_PENDING, test => {
      dataAnalyzer.analyze('test-pending', test)
    })

    runner.on(EventConsts.EVENT_TEST_END, test => {
      dataAnalyzer.analyze('test-end', test)
    })

    // 执行 失败 或 跳过
    runner.on(EventConsts.EVENT_TEST_FAIL, test => {
      if (
        test.err &&
        test.err instanceof Error &&
        test.err.message.startsWith('skiped')
        // (test.err.message.startsWith('skiped') ||
        //   test.err.message.includes(String(WebSkipedErrorCode)) ||
        //   test.err.message.includes(String(AppSkipedErrorCode))
        //   )
      ) {
        dataAnalyzer.analyze('test-skip', test)
      } else {
        dataAnalyzer.analyze('test-fail', test)
      }
    })

    // 执行成功
    runner.on(EventConsts.EVENT_TEST_PASS, test => {
      dataAnalyzer.analyze('test-pass', test)
    })

    // 测试完毕
    runner.on(EventConsts.EVENT_RUN_END, () => {
      dataAnalyzer.finish(runner)
    })
  }
}

export default QAReporter
