import { API_QA_STATUS } from '../constants'
import { apiMark } from '../qa-test-runner'
import { createStore, combineReducers } from 'redux'

/**
 * <apiList> -> [<apiInfo>]
 *
 * <apiInfo> ->
 * {
 *  status: enum,
 *  name: string,
 *  path: string,
 *  tests: [<testInfo>]
 *  id: string
 *  error: Error | null
 * }
 *
 * <testInfo> ->
 * {
 *  status: enum
 *  args: any []
 *  expectValue: any
 *  returnValue: any
 *  error: null | Error
 *  title: string
 * }
 */

function initRawApiList() {
  return Object.keys(apiMark).map(key => {
    const markItem = apiMark[key]
    const wrapperFn = markItem.value
    const namespaces = key.split('.')

    return {
      status: 0,
      name: namespaces.pop(),
      path: key,
      tests: [],
      ref: wrapperFn,
      scope: namespaces[0]
    }
  })
}

const defaultData = {
  rawApiList: initRawApiList()
}

function reducer(state = defaultData, action) {
  const { type, data } = action
  
  let testInfo = data
  let newList

  switch (type) {
    case 'UPDATE':
      newList = state.rawApiList.slice()
     
      const oldTestApiInfo = newList[testInfo.sort]
      const tests = oldTestApiInfo.tests

      tests.push(testInfo) // 收集测试结果

      // 当测试用例表现为异常时，更新ApiInfo的status
      if (
        testInfo.status === API_QA_STATUS.SKIP ||
        testInfo.status === API_QA_STATUS.FAIL
      ) {
        oldTestApiInfo.status = testInfo.status
      } else {
        oldTestApiInfo.status = API_QA_STATUS.PASS
      }

      oldTestApiInfo.tests = [...tests]
      newList[testInfo.sort] = { ...oldTestApiInfo }

      return { rawApiList: newList }
    case 'ADD':
      newList = state.rawApiList.slice()
      const rawItem = newList[testInfo.sort]

      rawItem.status = testInfo.status // 只需要更新apiInfo的状态就可以了

      newList[testInfo.sort] = { ...rawItem }

      return { rawApiList: newList }
    case 'RESET':
      return { rawApiList: initRawApiList() }

    default:
      return state
  }
}

export const stroe = createStore(combineReducers({ reducer }))
