import {
  API_QA_STATUS,
  API_QA_STATUS_TEXT,
} from '../../constants'
import { UI } from '@hyext/hy-ui'
import React, { Component } from 'react'
import "./index.hycss";
import { toString } from '../../utils/shared';

const {
  View,
  Text,
  ScrollView,
  NavigationBar,
} = UI

function renderJsonText(testInfo) {
  const { status, args, expectValue, returnValue, error, title} = testInfo
  
  switch (status) {
    case API_QA_STATUS.PEND:
    case API_QA_STATUS.SKIP:
      return null
    case API_QA_STATUS.FAIL:
      return (
        <>
          <JSONText data={title} name="标题"></JSONText>
          <JSONText data={API_QA_STATUS_TEXT[status]} name="状态"></JSONText>
          <JSONText data={args} name="入参"></JSONText>
          <JSONText data={expectValue} name="期望值"></JSONText>
          <JSONText data={returnValue} name="返回值"></JSONText>
          <JSONText data={error} name="错误"></JSONText>
        </>
      )
    case API_QA_STATUS.USER_PASS:
    case API_QA_STATUS.PASS:
      return (
        <>
          <JSONText data={title} name="标题"></JSONText>
          <JSONText data={API_QA_STATUS_TEXT[status]} name="状态"></JSONText>
          <JSONText data={args} name="入参"></JSONText>
          <JSONText data={expectValue} name="期望值"></JSONText>
          <JSONText data={returnValue} name="返回值"></JSONText>
        </>
      )
  }
}

function JSONText({ data, name }) {
  return (
    <View className="apiInfo-main">
      <Text className="apiInfo-title" numberOfLines={1}>
        {name}：
      </Text>
      <Text numberOfLines={9999}>{toString(data)}</Text>
    </View>
  )
}

function TestInfoPanel(props) {
  const testInfo = props.location.state

  return (
    <>
      <NavigationBar
        title={'测试结果'}
        onPressBack={() => {
          props.history.goBack()
        }}
      ></NavigationBar>
      <ScrollView className="apiInfo-scroller">
        <View className="apiInfo-panel">
          {renderJsonText(testInfo)}
        </View>
      </ScrollView>
    </>
  )
}

export default TestInfoPanel